package com.localmatters.collector.queue.consumer;

import com.localmatters.collector.queue.CollectorQueueException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;

/**
 * The collector consumer bean.
 *
 * @author Daniel Cavalcanti
 */
@WebService()
@Stateless()
public class ConsumerBean
    implements ConsumerRemote, ConsumerLocal {
    
    // **********************************************************************
    // Class variables
    // **********************************************************************
    
    /**
     * The logger
     */
    private static Logger logger = Logger.getLogger(ConsumerBean.class.getCanonicalName());
    
    // **********************************************************************
    // Instance variables
    // **********************************************************************
    
    /**
     * The JMS queue destination.
     */
    @Resource(mappedName="jms/CollectorQueue", name="CollectorQueue")
    private Queue queue;
    
    /**
     * The JMS connection factory.
     */
    @Resource(mappedName="jms/CollectorConnectionFactory")
    private ConnectionFactory connectionFactory;
    
    /**
     * Determines if the JMS session is transacted.
     */
    @Resource(name="session-transacted")
    private boolean transacted = false;
    
    /**
     * Determines the message acknowledge mode. If transacted is true, this
     * variable is ignored, and a {@link Session#SESSION_TRANSACTED} value is
     * assumed.
     */
    @Resource(name="session-ack-mode")
    private int acknowledgeMode = Session.SESSION_TRANSACTED;
    
    /**
     * The username used to connect to the queue.
     */
    @Resource(name="username")
    private String username;
    
    /**
     * The password used to connect to the queue.
     */
    @Resource(name="password")
    private String password;
    
    /**
     * The JMS connection.
     * Used to create a session into the collector queue.
     */
    private Connection connection;
    
    /**
     * The JMS session.
     * The session established to the collector queue.
     */
    private Session session;
    
    /**
     * The JMS message consumer.
     * A consumer to retrieve messages from the collector queue.
     */
    private MessageConsumer consumer;
    
    // **********************************************************************
    // Constructors
    // **********************************************************************
    
    /**
     * Creates a new instance of ConsumerBean.
     */
    public ConsumerBean() {
        logger.info("Creating consumer bean.");
    }
    
    // **********************************************************************
    // Lyfe-cycle methods
    // **********************************************************************
    
    /**
     * Initializes the JMS resources.
     */
    @PostConstruct()
    private void init() {
        
        logger.info("Initializing consumer bean.");
        logParams();
        
        try {
            
            logger.fine("Creating connection factory.");
            connection = connectionFactory.createConnection(username, password);
            
            logger.fine("Creating session.");
            session = connection.createSession(transacted, acknowledgeMode);
            
            logger.fine("Creating consumer.");
            consumer = session.createConsumer(queue);
            
            logger.fine("Starting connection.");
            connection.start();
            
            logger.info("Consumer bean initialized.");
            
        } catch (JMSException ex) {
            logger.log(Level.SEVERE, "Failed to initialize consumer bean.", ex);
        } catch (Throwable ex) {
            logger.log(Level.SEVERE, "Failed to initialize consumer bean.", ex);
        }
        
    }
    
    /**
     * Log producer bean parameters.
     */
    private void logParams() {
        
        final String endl = System.getProperty("line.separator", "\n");
        StringBuilder buf = new StringBuilder();
        
        buf.append("Consumer bean parameters.").append(endl);
        buf.append("Session transacted..: ").append(transacted).append(endl);
        buf.append("Acknowledgement mode: ").append(acknowledgeMode).append(endl);
        buf.append("Queue username......: ").append(username).append(endl);
        buf.append("Queue password......: ").append(password).append(endl);
        
        logger.config(buf.toString());
        
    }
    
    /**
     * Release the JMS resources.
     */
    @PreDestroy()
    private void release() {
        
        logger.info("Consumer bean releasing resources.");
        
        try {
            
            logger.info("Stopping connection.");
            connection.stop();
            logger.info("Connection stopped.");
            
        } catch (JMSException ex1) {
            logger.log(Level.SEVERE, "Failed to stop connection.", ex1);
        } catch (Throwable ex1) {
            logger.log(Level.SEVERE, "Failed to stop connection.", ex1);
        } finally {
            
            try {
                
                logger.info("Closing connection.");
                connection.close();
                logger.info("Connection closed.");
                
            } catch (JMSException ex2) {
                logger.log(Level.SEVERE, "Failed to close connection.", ex2);
            } catch (Throwable ex2) {
                logger.log(Level.SEVERE, "Failed to close connection.", ex2);
            }
            
        }
        
    }
    
    // **********************************************************************
    // ConsumerService methods
    // **********************************************************************
    
    @WebMethod()
    public String retrieve()
    throws CollectorQueueException {
        
        try {
            
            logger.info("Retrieving message.");
            
            Message msg = consumer.receive();
            Object obj = ((ObjectMessage) msg).getObject();
            
            logger.info("Message retrieved.");
            logger.fine(obj.toString());
            
            return (String) obj;
            
        } catch (JMSException ex) {
            
            logger.log(Level.SEVERE, "Failed to retrieve message.", ex);
            throw new CollectorQueueException(ex);
            
        } catch (Throwable ex) {
            
            logger.log(Level.SEVERE, "Failed to retrieve message.", ex);
            throw new CollectorQueueException(ex);
            
        }
        
    }
    
    @WebMethod()
    public String retrieveTimed(@WebParam() long timeout)
    throws CollectorQueueException {
        
        try {
            
            logger.info("Retrieving message with timeout: " + timeout + "ms.");
            
            Message msg = consumer.receive(timeout);
            
            if (msg == null) {
                logger.info("No message in queue. Retrieve timed out.");
                return null;
            }
            
            Object obj = ((ObjectMessage) msg).getObject();
            
            logger.info("Message retrieved.");
            logger.fine(obj.toString());
            
            return (String) obj;
            
            
        } catch (JMSException ex) {
            
            logger.log(Level.SEVERE, "Failed to retrieve message.", ex);
            throw new CollectorQueueException(ex);
            
        } catch (Throwable ex) {
            
            logger.log(Level.SEVERE, "Failed to retrieve message.", ex);
            throw new CollectorQueueException(ex);
            
        }
        
    }
    
    @WebMethod()
    public String retrieveImmediate()
    throws CollectorQueueException {
        
        try {
            
            logger.info("Retrieving message immediately.");
            
            Message msg = consumer.receiveNoWait();
            
            if (msg == null) {
                logger.info("No message in queue.");
                return null;
            }
            
            Object obj = ((ObjectMessage) msg).getObject();
            
            logger.info("Message retrieved.");
            logger.fine(obj.toString());
            
            return (String) obj;
            
        } catch (JMSException ex) {
            
            logger.log(Level.SEVERE, "Failed to retrieve message.", ex);
            throw new CollectorQueueException(ex);
            
        } catch (Throwable ex) {
            
            logger.log(Level.SEVERE, "Failed to retrieve message.", ex);
            throw new CollectorQueueException(ex);
            
        }
        
    }
    
}
