package com.localmatters.collector.queue.producer;

import com.localmatters.collector.queue.CollectorQueueException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;

/**
 * The collector producer bean.
 *
 * @author Daniel Cavalcanti
 */
@Stateless()
@WebService()
public class ProducerBean
    implements ProducerRemote, ProducerLocal {
    
    // **********************************************************************
    // Class variables
    // **********************************************************************
    
    /**
     * The logger
     */
    private static Logger logger = Logger.getLogger(ProducerBean.class.getCanonicalName());
    
    // **********************************************************************
    // Instance variables
    // **********************************************************************
    
    /**
     * The JMS queue destination.
     */
    @Resource(mappedName="jms/CollectorQueue", name="CollectorQueue")
    private Queue queue;
    
    /**
     * The JMS connection factory.
     */
    @Resource(mappedName="jms/CollectorConnectionFactory")
    private ConnectionFactory connectionFactory;
    
    /**
     * Determines if the JMS session is transacted.
     */
    @Resource(name="session-transacted")
    private boolean transacted = false;
    
    /**
     * Determines the message acknowledge mode. If transacted is true, this
     * variable is ignored, and a {@link Session#SESSION_TRANSACTED} value is
     * assumed.
     */
    @Resource(name="session-ack-mode")
    private int acknowledgeMode = Session.SESSION_TRANSACTED;
    
    /**
     * The username used to connect to the queue.
     */
    @Resource(name="username")
    private String username;
    
    /**
     * The password used to connect to the queue.
     */
    @Resource(name="password")
    private String password;
    
    /**
     * The JMS connection.
     * Used to create a session into the collector queue.
     */
    private Connection connection;
    
    /**
     * The JMS session.
     * The session established to the collector queue.
     */
    private Session session;
    
    /**
     * The JMS message producer.
     * A producer to place messages into the collector queue.
     */
    private MessageProducer producer;
    
    // **********************************************************************
    // Constructors
    // **********************************************************************
    
    /**
     * Creates a new instance of ProducerBean.
     */
    public ProducerBean() {
        logger.info("Creating producer bean.");
    }
    
    // **********************************************************************
    // Lyfe-cycle methods
    // **********************************************************************
    
    /**
     * Initializes the JMS resources.
     */
    @PostConstruct()
    private void init() {
        
        logger.info("Initializing producer bean.");
        logParams();
        
        try {
            
            logger.fine("Creating connection factory.");
            connection = connectionFactory.createConnection(username, password);
            
            logger.fine("Creating session.");
            session = connection.createSession(transacted, acknowledgeMode);
            
            logger.fine("Creating producer.");
            producer = session.createProducer(queue);
            
            logger.fine("Starting connection.");
            connection.start();
            
            logger.info("Producer bean initialized.");
            
        } catch (JMSException ex) {
            logger.log(Level.SEVERE, "Failed to initialize producer bean.", ex);
        } catch (Throwable ex) {
            logger.log(Level.SEVERE, "Failed to initialize producer bean.", ex);
        }
        
    }
    
    /**
     * Log producer bean parameters.
     */
    private void logParams() {
        
        final String endl = System.getProperty("line.separator", "\n");
        StringBuilder buf = new StringBuilder();
        
        buf.append("Producer bean parameters.").append(endl);
        buf.append("Session transacted..: ").append(transacted).append(endl);
        buf.append("Acknowledgement mode: ").append(acknowledgeMode).append(endl);
        buf.append("Queue username......: ").append(username).append(endl);
        buf.append("Queue password......: ").append(password).append(endl);
        
        logger.config(buf.toString());
        
    }
    
    /**
     * Release the JMS resources.
     */
    @PreDestroy()
    private void release() {
        
        logger.info("Producer bean releasing resources.");
        
        try {
            
            logger.info("Stopping connection.");
            connection.stop();
            logger.info("Connection stopped.");
            
        } catch (JMSException ex1) {
            logger.log(Level.SEVERE, "Failed to stop connection.", ex1);
        } catch (Throwable ex1) {
            logger.log(Level.SEVERE, "Failed to stop connection.", ex1);
        } finally {
            
            try {
                
                logger.info("Closing connection.");
                connection.close();
                logger.info("Connection closed.");
                
            } catch (JMSException ex2) {
                logger.log(Level.SEVERE, "Failed to close connection.", ex2);
            } catch (Throwable ex2) {
                logger.log(Level.SEVERE, "Failed to close connection.", ex2);
            }
            
        }
        
    }
    
    // **********************************************************************
    // ProducerService methods
    // **********************************************************************
    
    @WebMethod()
    public void send(@WebParam() String message)
    throws CollectorQueueException {
        
        try {
            
            logger.info("Sending message.");
            logger.fine(message);
            
            producer.send(session.createObjectMessage(message));
            
            logger.info("Message sent.");
            
        } catch (JMSException ex) {
            
            logger.log(Level.SEVERE, "Failed to send message.", ex);
            throw new CollectorQueueException(ex);
            
        } catch (Throwable ex) {
            
            logger.log(Level.SEVERE, "Failed to send message.", ex);
            throw new CollectorQueueException(ex);
            
        }
        
    }
    
}
